from yby.y_control import y_start
y_start()


##############################################################################

def y_txt2csv(txt_name, delimiter = ','):

    '''
    Transform txt file to csv file

    PARAMETERS:
    -----------
    txt_name  -  filename of txt
    delimiter -  comma is default

    RETURNS:
    --------
    generate a csv file that has the same name with txt file
    '''
    
    import csv
    
    new_name = txt_name[:-3] + 'csv'
    with open(txt_name, 'r') as fp1:
        content = []
        for line in fp1.readlines():
            line = line.strip().split(delimiter) #删除每行的空格和定界符
            content.append(line)
        with open(new_name, 'w', newline = '') as fp2: #创建一个同名的csv文件
            w = csv.writer(fp2)
            w.writerows(content)

##############################################################################
import json

class MyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return int(obj)
        elif isinstance(obj, np.floating):
            return float(obj)
        elif isinstance(obj, np.ndarray):
            return obj.tolist()
        if isinstance(obj, time):
            return obj.__str__()
        else:
            return super(NpEncoder, self).default(obj)
    
def todict(matobj):
    
    '''
    A recursive function which constructs from matobjects nested dictionaries

    PARAMETERS:
    -----------
    matobj    - matlab object, one element

    RETURNS:
    --------
    dict_tem  - tem dict of a matobj(one key, one step, one element)
    '''
    dict_tem = {}
    for strg in matobj._fieldnames:
        elem = matobj.__dict__[strg]
        if isinstance(elem, spio.matlab.mio5_params.mat_struct):
            dict_tem[strg] = todict(elem)
        else:
            dict_tem[strg] = elem
    return dict_tem

def y_mat2json(mat_file, file_gen ='off', json_file=None, verbose='off'):
    
    '''
    convert mat file to json format
    
    PARAMETERS:
    ----------
    mat_file   - abspath of mat file
    file_gen   - default is 'off', generate a json file or not
    json_file  - default is None, otherwise generate a json file at json_file
    verbose    - default is 'off', otherwise display json_data.keys() infomation
    
    RETURNS:
    --------
    return     - json_data
    '''
    
    import os
    import json
    import scipy.io as spio
    import numpy as np
    import pandas as pd


    # load data

    data = spio.loadmat(mat_file, struct_as_record=False, squeeze_me=True)

    # updata data(remove header, version, globals)

    data.pop('__header__')
    data.pop('__version__')
    data.pop('__globals__')

    # classified variables and record in type_v

    type_v = {'dict_1':[], 'dict_2':[], 'dict_3':[], 'dict_4':[], 'general':[]}
    for key in data.keys():
        if isinstance(data[key], spio.matlab.mio5_params.mat_struct):
            type_v['dict_1'].append(key)
        elif key=='W':
            type_v['dict_4'].append(key)
        elif isinstance(data[key], np.ndarray) and isinstance(data[key][0], spio.matlab.mio5_params.mat_struct) and key!='W':
            type_v['dict_2'].append(key)
        elif isinstance(data[key], np.ndarray) and isinstance(data[key][0], np.ndarray) and isinstance(data[key][0][0], spio.matlab.mio5_params.mat_struct):
            type_v['dict_3'].append(key)
        else:
            type_v['general'].append(key)
            
    # process general

    dict_general = {}
    for key in type_v['general']:
        dict_general[key] = data[key]
        
    # process dict_1: value is matobj

    dict_1 = {}
    for key in type_v['dict_1']:
        dict_1[key] = todict(data[key])

    # process dict_2: struct contains step value cells, value is array, array elem is matobj

    dict_2 = {}
    for key in type_v['dict_2']:
        dict_step = {}
        for s in range(len(data[key])):
            dict_step['step'+str(+1)] = todict(data[key][s])
        dict_2[key] = dict_step

    # process dict_3: struct contains step value cell, and cell is 1*n
    dict_3 = {}
    for key in type_v['dict_3']:
        dict_step = {}
        for s in range(len(data[key])):
            dict_well = {}
            for well in range(len(data[key][s])):
                dict_well['well'+str(well+1)] = todict(data[key][s][well])
            dict_step['step'+str(s+1)] = dict_well
        dict_3[key] = dict_step

    # process dict_4: for 'W' which is similar to dict_2
    dict_4 = {}
    for key in type_v['dict_4']:
        dict_step = {}
        for s in range(len(data[key])):
            dict_step['well'+str(s+1)] = todict(data[key][s])
        dict_4[key] = dict_step
        
    # combine all types dict

    dict_all = dict(dict_general, **dict_1, **dict_2, **dict_3, **dict_4)
    dict_all['type'] = type_v

    # file_gen

    if file_gen == 'on':
        if json_file:
            with open(json_file, 'w') as f:
                json.dump(dict_all, f, cls='MyEncoder')
        else:
            json_file = mat_file[:-3] + 'json'
            with open(json_file, 'w') as f:
                json.dump(dict_all, f, cls='MyEncoder')
    
    if verbose=='on':
        print(dict_all['type'])
        
    return dict_all

###############################################################################

